@@ -1,8 +1,7 @@
-import socket
+import json
 import webbrowser
 import httplib2
 import oauth2client.clientsecrets as clientsecrets
-from six.moves import input
 import threading
 
 from googleapiclient.discovery import build
@@ -12,6 +11,7 @@ from oauth2client.client import FlowExchangeError
 from oauth2client.client import AccessTokenRefreshError
 from oauth2client.client import OAuth2WebServerFlow
 from oauth2client.client import OOB_CALLBACK_URN
+from oauth2client.contrib.dictionary_storage import DictionaryStorage
 from oauth2client.file import Storage
 from oauth2client.tools import ClientRedirectHandler
 from oauth2client.tools import ClientRedirectServer
@@ -141,7 +141,7 @@ def CheckAuth(decoratee):
     return _decorated
 
 
-class GoogleAuth(ApiAttributeMixin, object):
+class GoogleAuth(ApiAttributeMixin):
     """Wrapper class for oauth2client library in google-api-python-client.
 
     Loads all settings and credentials from one 'settings.yaml' file
@@ -172,7 +172,9 @@ class GoogleAuth(ApiAttributeMixin, object):
     service = ApiAttribute("service")
     auth_method = ApiAttribute("auth_method")
 
-    def __init__(self, settings_file="settings.yaml", http_timeout=None):
+    def __init__(
+        self, settings_file="settings.yaml", http_timeout=None, settings=None
+    ):
         """Create an instance of GoogleAuth.
 
         This constructor just sets the path of settings file.
@@ -180,23 +182,26 @@ class GoogleAuth(ApiAttributeMixin, object):
 
         :param settings_file: path of settings file. 'settings.yaml' by default.
         :type settings_file: str.
+        :param settings: settigs dict.
+        :type settings: dict.
         """
         self.http_timeout = http_timeout
         ApiAttributeMixin.__init__(self)
         self.thread_local = threading.local()
         self.client_config = {}
-        try:
-            self.settings = LoadSettingsFile(settings_file)
-        except SettingsError:
-            self.settings = self.DEFAULT_SETTINGS
-        else:
-            if self.settings is None:
-                self.settings = self.DEFAULT_SETTINGS
-            else:
-                ValidateSettings(self.settings)
-        self._storages = self._InitializeStoragesFromSettings()
-        # Only one (`file`) backend is supported now
-        self._default_storage = self._storages["file"]
+
+        if settings is None and settings_file:
+            try:
+                settings = LoadSettingsFile(settings_file)
+            except SettingsError:
+                pass
+
+        self.settings = settings or self.DEFAULT_SETTINGS
+        ValidateSettings(self.settings)
+
+        storages, default = self._InitializeStoragesFromSettings()
+        self._storages = storages
+        self._default_storage = default
 
     @property
     def access_token_expired(self):
@@ -241,13 +246,13 @@ class GoogleAuth(ApiAttributeMixin, object):
                 httpd = ClientRedirectServer(
                     (host_name, port), ClientRedirectHandler
                 )
-            except socket.error:
+            except OSError:
                 pass
             else:
                 success = True
                 break
         if success:
-            oauth_callback = "http://%s:%s/" % (host_name, port_number)
+            oauth_callback = f"http://{host_name}:{port_number}/"
         else:
             print(
                 "Failed to start a local web server. Please check your firewall"
@@ -304,11 +309,24 @@ class GoogleAuth(ApiAttributeMixin, object):
         if set(self.SERVICE_CONFIGS_LIST) - set(self.client_config):
             self.LoadServiceConfigSettings()
         scopes = scopes_to_string(self.settings["oauth_scope"])
-        client_service_json = self.client_config.get("client_json_file_path")
-        if client_service_json:
+        keyfile_name = self.client_config.get("client_json_file_path")
+        keyfile_dict = self.client_config.get("client_json_dict")
+        keyfile_json = self.client_config.get("client_json")
+
+        if not keyfile_dict and keyfile_json:
+            # Compensating for missing ServiceAccountCredentials.from_json_keyfile
+            keyfile_dict = json.loads(keyfile_json)
+
+        if keyfile_dict:
+            self.credentials = (
+                ServiceAccountCredentials.from_json_keyfile_dict(
+                    keyfile_dict=keyfile_dict, scopes=scopes
+                )
+            )
+        elif keyfile_name:
             self.credentials = (
                 ServiceAccountCredentials.from_json_keyfile_name(
-                    filename=client_service_json, scopes=scopes
+                    filename=keyfile_name, scopes=scopes
                 )
             )
         else:
@@ -327,7 +345,7 @@ class GoogleAuth(ApiAttributeMixin, object):
             )
 
     def _InitializeStoragesFromSettings(self):
-        result = {"file": None}
+        result = {"file": None, "dictionary": None}
         backend = self.settings.get("save_credentials_backend")
         save_credentials = self.settings.get("save_credentials")
         if backend == "file":
@@ -337,11 +355,21 @@ class GoogleAuth(ApiAttributeMixin, object):
                     "Please specify credentials file to read"
                 )
             result[backend] = Storage(credentials_file)
+        elif backend == "dictionary":
+            creds_dict = self.settings.get("save_credentials_dict")
+            if creds_dict is None:
+                raise InvalidConfigError("Please specify credentials dict")
+
+            creds_key = self.settings.get("save_credentials_key")
+            if creds_key is None:
+                raise InvalidConfigError("Please specify credentials key")
+
+            result[backend] = DictionaryStorage(creds_dict, creds_key)
         elif save_credentials:
             raise InvalidConfigError(
                 "Unknown save_credentials_backend: %s" % backend
             )
-        return result
+        return result, result.get(backend)
 
     def LoadCredentials(self, backend=None):
         """Loads credentials or create empty credentials if it doesn't exist.
@@ -356,6 +384,8 @@ class GoogleAuth(ApiAttributeMixin, object):
                 raise InvalidConfigError("Please specify credential backend")
         if backend == "file":
             self.LoadCredentialsFile()
+        elif backend == "dictionary":
+            self._LoadCredentialsDictionary()
         else:
             raise InvalidConfigError("Unknown save_credentials_backend")
 
@@ -381,7 +411,7 @@ class GoogleAuth(ApiAttributeMixin, object):
 
         try:
             self.credentials = self._default_storage.get()
-        except IOError:
+        except OSError:
             raise InvalidCredentialsError(
                 "Credentials file cannot be symbolic link"
             )
@@ -389,6 +419,19 @@ class GoogleAuth(ApiAttributeMixin, object):
         if self.credentials:
             self.credentials.set_store(self._default_storage)
 
+    def _LoadCredentialsDictionary(self):
+        self._default_storage = self._storages["dictionary"]
+        if self._default_storage is None:
+            raise InvalidConfigError(
+                "Backend `dictionary` is not configured, specify "
+                "credentials dict and key to read in the settings file"
+            )
+
+        self.credentials = self._default_storage.get()
+
+        if self.credentials:
+            self.credentials.set_store(self._default_storage)
+
     def SaveCredentials(self, backend=None):
         """Saves credentials according to specified backend.
 
@@ -405,6 +448,8 @@ class GoogleAuth(ApiAttributeMixin, object):
                 raise InvalidConfigError("Please specify credential backend")
         if backend == "file":
             self.SaveCredentialsFile()
+        elif backend == "dictionary":
+            self._SaveCredentialsDictionary()
         else:
             raise InvalidConfigError("Unknown save_credentials_backend")
 
@@ -431,11 +476,24 @@ class GoogleAuth(ApiAttributeMixin, object):
 
         try:
             storage.put(self.credentials)
-        except IOError:
+        except OSError:
             raise InvalidCredentialsError(
                 "Credentials file cannot be symbolic link"
             )
 
+    def _SaveCredentialsDictionary(self):
+        if self.credentials is None:
+            raise InvalidCredentialsError("No credentials to save")
+
+        storage = self._storages["dictionary"]
+        if storage is None:
+            raise InvalidConfigError(
+                "Backend `dictionary` is not configured, specify "
+                "credentials dict and key to write in the settings file"
+            )
+
+        storage.put(self.credentials)
+
     def LoadClientConfig(self, backend=None):
         """Loads client configuration according to specified backend.
 
@@ -513,22 +571,27 @@ class GoogleAuth(ApiAttributeMixin, object):
             pass  # The service auth fields are not present, handling code can go here.
 
     def LoadServiceConfigSettings(self):
-        """Loads client configuration from settings file.
+        """Loads client configuration from settings.
         :raises: InvalidConfigError
         """
-        for file_format in ["json", "pkcs12"]:
-            config = f"client_{file_format}_file_path"
+        configs = [
+            "client_json_file_path",
+            "client_json_dict",
+            "client_json",
+            "client_pkcs12_file_path",
+        ]
+
+        for config in configs:
             value = self.settings["service_config"].get(config)
             if value:
                 self.client_config[config] = value
                 break
         else:
             raise InvalidConfigError(
-                "Either json or pkcs12 file path required "
-                "for service authentication"
+                f"One of {configs} is required for service authentication"
             )
 
-        if file_format == "pkcs12":
+        if config == "client_pkcs12_file_path":
             self.SERVICE_CONFIGS_LIST.append("client_service_email")
 
         for config in self.SERVICE_CONFIGS_LIST:
@@ -538,7 +601,7 @@ class GoogleAuth(ApiAttributeMixin, object):
                 ]
             except KeyError:
                 err = "Insufficient service config in settings"
-                err += "\n\nMissing: {} key.".format(config)
+                err += f"\n\nMissing: {config} key."
                 raise InvalidConfigError(err)
 
     def LoadClientConfigSettings(self):
