@@ -17,9 +17,7 @@ from .auth import LoadAuth
 BLOCK_SIZE = 1024
 # Usage: MIME_TYPE_TO_BOM['<Google Drive mime type>']['<download mimetype>'].
 MIME_TYPE_TO_BOM = {
-    "application/vnd.google-apps.document": {
-        "text/plain": "\ufeff".encode("utf8")
-    }
+    "application/vnd.google-apps.document": {"text/plain": "\ufeff".encode()}
 }
 
 
@@ -68,7 +66,7 @@ class GoogleDriveFileList(ApiResourceList):
 
     def __init__(self, auth=None, param=None):
         """Create an instance of GoogleDriveFileList."""
-        super(GoogleDriveFileList, self).__init__(auth=auth, metadata=param)
+        super().__init__(auth=auth, metadata=param)
 
     @LoadAuth
     def _GetList(self):
@@ -98,7 +96,7 @@ class GoogleDriveFileList(ApiResourceList):
         return result
 
 
-class IoBuffer(object):
+class IoBuffer:
     """Lightweight retention of one chunk."""
 
     def __init__(self, encoding):
@@ -116,7 +114,7 @@ class IoBuffer(object):
         )
 
 
-class MediaIoReadable(object):
+class MediaIoReadable:
     def __init__(
         self,
         request,
@@ -297,6 +295,7 @@ class GoogleDriveFile(ApiAttributeMixin, ApiResource):
         remove_bom=False,
         callback=None,
         chunksize=DEFAULT_CHUNK_SIZE,
+        acknowledge_abuse=False,
     ):
         """Save content of this file as a local file.
 
@@ -310,6 +309,9 @@ class GoogleDriveFile(ApiAttributeMixin, ApiResource):
         :type param: callable
         :param chunksize: chunksize in bytes (standard 100 MB(1024*1024*100))
         :type chunksize: int
+        :param acknowledge_abuse: Acknowledging the risk and download file
+            identified as abusive.
+        :type acknowledge_abuse: bool
         :raises: ApiRequestError, FileNotUploadedError
         """
         files = self.auth.service.files()
@@ -333,7 +335,12 @@ class GoogleDriveFile(ApiAttributeMixin, ApiResource):
             # But that would first require a slow call to FetchMetadata().
             # We prefer to try-except for speed.
             try:
-                download(fd, files.get_media(fileId=file_id))
+                download(
+                    fd,
+                    files.get_media(
+                        fileId=file_id, acknowledgeAbuse=acknowledge_abuse
+                    ),
+                )
             except errors.HttpError as error:
                 exc = ApiRequestError(error)
                 if (
@@ -364,6 +371,7 @@ class GoogleDriveFile(ApiAttributeMixin, ApiResource):
         encoding=None,
         remove_bom=False,
         chunksize=DEFAULT_CHUNK_SIZE,
+        acknowledge_abuse=False,
     ):
         """Get a file-like object which has a buffered read() method.
 
@@ -375,6 +383,9 @@ class GoogleDriveFile(ApiAttributeMixin, ApiResource):
         :type remove_bom: bool
         :param chunksize: default read()/iter() chunksize.
         :type chunksize: int
+        :param acknowledge_abuse: Acknowledging the risk and download file
+            identified as abusive.
+        :type acknowledge_abuse: bool
         :returns: MediaIoReadable -- file-like object.
         :raises: ApiRequestError, FileNotUploadedError
         """
@@ -388,7 +399,11 @@ class GoogleDriveFile(ApiAttributeMixin, ApiResource):
         # But that would first require a slow call to FetchMetadata().
         # We prefer to try-except for speed.
         try:
-            request = self._WrapRequest(files.get_media(fileId=file_id))
+            request = self._WrapRequest(
+                files.get_media(
+                    fileId=file_id, acknowledgeAbuse=acknowledge_abuse
+                )
+            )
             return MediaIoReadable(
                 request, encoding=encoding, chunksize=chunksize
             )
@@ -520,6 +535,42 @@ class GoogleDriveFile(ApiAttributeMixin, ApiResource):
         """
         self._FilesDelete(param=param)
 
+    @LoadAuth
+    def Copy(self, target_folder=None, new_title=None, param=None):
+        """Creates a copy of this file. Folders cannot be copied.
+
+        :param target_folder: Folder where the file will be copied.
+        :type target_folder: GoogleDriveFile, optional
+        :param new_title: Name of the new file.
+        :type new_title: str, optional
+        :param param: addition parameters to pass.
+        :type param: dict, optional
+        :raises: ApiRequestError
+        :return: the copied file
+        :rtype: GoogleDriveFile
+        """
+
+        if param is None:
+            param = {}
+
+        param["fileId"] = self["id"]
+        param["supportsAllDrives"] = True
+        param["body"] = {}
+
+        if target_folder:
+            param["body"]["parents"] = [{"id": target_folder["id"]}]
+        param["body"]["title"] = new_title
+
+        new_file = None
+        try:
+            new_file = (
+                self.auth.service.files().copy(**param).execute(http=self.http)
+            )
+        except errors.HttpError as error:
+            raise ApiRequestError(error)
+
+        return GoogleDriveFile(self.auth, new_file)
+
     def InsertPermission(self, new_permission, param=None):
         """Insert a new permission. Re-fetches all permissions after call.
 
