@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 import filecmp
 import os
 import unittest
@@ -8,7 +7,6 @@ from io import BytesIO
 from tempfile import mkdtemp
 from time import time
 
-from six.moves import range
 import timeout_decorator
 from concurrent.futures import ThreadPoolExecutor, as_completed
 from googleapiclient import errors
@@ -699,22 +697,22 @@ class GoogleDriveFileTest(unittest.TestCase):
 
     def test_InsertPrefix(self):
         # Create BytesIO.
-        file_obj = BytesIO("abc".encode("utf8"))
+        file_obj = BytesIO(b"abc")
         original_length = len(file_obj.getvalue())
-        char_to_insert = "\ufeff".encode("utf8")
+        char_to_insert = "\ufeff".encode()
 
         # Insert the prefix.
         GoogleDriveFile._InsertPrefix(file_obj, char_to_insert)
         modified_length = len(file_obj.getvalue())
         self.assertGreater(modified_length, original_length)
-        self.assertEqual(file_obj.getvalue(), "\ufeffabc".encode("utf8"))
+        self.assertEqual(file_obj.getvalue(), "\ufeffabc".encode())
 
     def test_InsertPrefixLarge(self):
         # Create BytesIO.
         test_content = "abc" * 800
         file_obj = BytesIO(test_content.encode("utf-8"))
         original_length = len(file_obj.getvalue())
-        char_to_insert = "\ufeff".encode("utf8")
+        char_to_insert = "\ufeff".encode()
 
         # Insert the prefix.
         GoogleDriveFile._InsertPrefix(file_obj, char_to_insert)
@@ -725,22 +723,22 @@ class GoogleDriveFileTest(unittest.TestCase):
 
     def test_RemovePrefix(self):
         # Create BytesIO.
-        file_obj = BytesIO("\ufeffabc".encode("utf8"))
+        file_obj = BytesIO("\ufeffabc".encode())
         original_length = len(file_obj.getvalue())
-        char_to_remove = "\ufeff".encode("utf8")
+        char_to_remove = "\ufeff".encode()
 
         # Insert the prefix.
         GoogleDriveFile._RemovePrefix(file_obj, char_to_remove)
         modified_length = len(file_obj.getvalue())
         self.assertLess(modified_length, original_length)
-        self.assertEqual(file_obj.getvalue(), "abc".encode("utf8"))
+        self.assertEqual(file_obj.getvalue(), b"abc")
 
     def test_RemovePrefixLarge(self):
         # Create BytesIO.
         test_content = "\ufeff" + "abc" * 800
         file_obj = BytesIO(test_content.encode("utf8"))
         original_length = len(file_obj.getvalue())
-        char_to_remove = "\ufeff".encode("utf8")
+        char_to_remove = "\ufeff".encode()
 
         # Insert the prefix.
         GoogleDriveFile._RemovePrefix(file_obj, char_to_remove)
@@ -844,6 +842,75 @@ class GoogleDriveFileTest(unittest.TestCase):
         files = self._parallel_uploader(self.FILE_UPLOAD_COUNT, 10)
         self._parallel_downloader(files, 10)
 
+    # Tests for Copy file.
+    # ====================
+
+    def test_CopyFileSameFolder(self):
+        drive = GoogleDrive(self.ga)
+
+        content = "hello world!"
+
+        # create a temp file and set it's content to a known string
+        file1 = drive.CreateFile()
+        filename = self.getTempFile("copytestfile", content=content)
+        file1["title"] = filename
+        file1.SetContentFile(filename)
+        pydrive_retry(file1.Upload)
+
+        # copy the file
+        file2 = pydrive_retry(file1.Copy, new_title="copytestfile_copy")
+
+        self.assertIsNotNone(file2)
+
+        pydrive_retry(file2.FetchContent)
+        # assert that the content of the copied file is the same as the original (file1)
+        self.assertEqual(file2.GetContentString(), content)
+
+        self.DeleteUploadedFiles(drive, [file1["id"], file2["id"]])
+
+    def test_CopyFileDifferentFolder(self):
+        drive = GoogleDrive(self.ga)
+
+        content = "hello world!"
+
+        # create a temp file and set it's content to a known string
+        file1 = drive.CreateFile()
+        filename = self.getTempFile("copytestfile", content=content)
+        file1["title"] = filename
+        file1.SetContentFile(filename)
+        pydrive_retry(file1.Upload)
+
+        # create a temp directory
+        temp_dir = pydrive_retry(
+            drive.CreateFile,
+            {
+                "title": "temp_dir",
+                "mimeType": "application/vnd.google-apps.folder",
+                "parents": [{"id": file1["parents"][0]["id"]}],
+            },
+        )
+        pydrive_retry(temp_dir.Upload)
+
+        # copy the file into the new folder
+        file2 = pydrive_retry(
+            file1.Copy, target_folder=temp_dir, new_title="copytestfile_copy"
+        )
+
+        self.assertIsNotNone(file2)
+
+        pydrive_retry(file2.FetchContent)
+        # assert that the content of the copied file is the same as the original (file1)
+        self.assertEqual(file2.GetContentString(), content)
+
+        files = pydrive_retry(
+            drive.ListFile, {"q": f"'{temp_dir['id']}' in parents"}
+        ).GetList()
+        self.assertIn("copytestfile_copy", [f["title"] for f in files])
+
+        self.DeleteUploadedFiles(
+            drive, [file1["id"], file2["id"], temp_dir["id"]]
+        )
+
     # Helper functions.
     # =================
 
